# GNU Solfege - ear training for GNOME
# Copyright (C) 2005  Tom Cato Amundsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
This file does the checking for optional features that depend on
python modules the user can have installed.

It also does sanity check on the python and pygtk versions and some
other initial setup tasks.
"""

import sys
import os

_has_gtkhtml2 = None
_has_gnome_canvas = None
_has_gnome = None

def assert_python_version(required_version):
    if sys.version_info < required_version:
        print "*"* 67
        print "Solfege need Python %s or newer. The configure script told you so!" % (".".join([str(i) for i in required_version]))
        print "*"* 67
        sys.exit(-1)

def setup_pygtk(required_version):
    import pygtk
    # this is needed for py2exe
    if sys.platform == 'win32':
        os.environ['PATH'] += ";lib;bin;"
    else:
        pygtk.require("2.0")
    import gtk
    gtk.rc_parse("solfege.gtkrc")
    # The rest of this function is just for sanity check. Not really required.
    if gtk.pygtk_version < required_version:
        print "-"*55
        print " GNU Solfege requires pygtk version %s or newer." % (".".join([str(i) for i in required_version]))
        print " The version installed appears to be %s" % (".".join([str(i) for i in gtk.pygtk_version]))
        print " Exiting program."
        print "-"*55+"\n"
        sys.exit(-1)


def init(options):
    global _has_gtkhtml2
    global _has_gnome
    assert_python_version((2, 3))
    setup_pygtk((2, 4))
    if options.without_gtkhtml:
        _has_gtkhtml2 = False
        print "gtkhtml2: disabled by runtime option"
    if options.without_gnome:
        _has_gnome = False
        print "gnome: disabled by runtime option"



def has_gtkhtml2():
    global _has_gtkhtml2
    if _has_gtkhtml2 is not None:
        return _has_gtkhtml2
    print "Checking for gtkhtml2...",
    try:
        import gtkhtml2
        print "ok"
        _has_gtkhtml2 = True
    except ImportError, e:
        print e
        _has_gtkhtml2 = False
    return _has_gtkhtml2

def has_gnome_canvas():
    global _has_gnome_canvas
    if _has_gnome_canvas is not None:
        return _has_gnome_canvas
    print "Checking for gnome.canvas",
    try:
        import gnome.canvas
        try:
            gnome.canvas.not_real_canvas
        except:
            print "ok"
            _has_gnome_canvas = True
    except ImportError, e:
        print e
        _has_gnome_canvas = False
    return _has_gnome_canvas

def has_gnome():
    global _has_gnome
    if _has_gnome is not None:
        return _has_gnome
    print "Checking for gnome",
    try:
        import gnome
        print "ok"
        _has_gnome = True
    except ImportError, e:
        print e
        _has_gnome = False
    return _has_gnome
